### Name: pSegment
### Title: Parallelized versions of several aCGH segementation
###   algorithms/methods
### Aliases: pSegmentDNAcopy pSegmentACE pSegmentHMM pSegmentGLAD
###   pSegmentBioHMM pSegmentCGHseg pSegmentPSW pSegmentWavelets pSegment
### Keywords: nonparametric

### ** Examples

## Not run: 
##D setwd("/tmp/o3") ## all slaves need a common dir to read and write.
##D mpiInit()
## End(Not run)

## library(Rmpi)
## try(mpi.spawn.Rslaves(nslaves = 1), silent = TRUE)

data(cghE1)
tmpchr <- sub("chr", "", cghE1$Chromosome)
chrom.numeric <- as.numeric(as.character(tmpchr))
chrom.numeric[tmpchr == "X"] <- 23
chrom.numeric[tmpchr == "Y"] <- 24
rm(tmpchr)
### we need the data ordered
reorder <- order(chrom.numeric,
                 cghE1$UG.Start,
                 cghE1$UG.End,
                 cghE1$Name)
cghE1 <- cghE1[reorder, ]
chrom.numeric <- chrom.numeric[reorder]

## run all methods
hmm.out <- pSegmentHMM(cghE1[, 5:7], chrom.numeric)
glad.out <- pSegmentGLAD(cghE1[, 5:7], chrom.numeric)
cghseg.out <- pSegmentCGHseg(cghE1[, 5:7], chrom.numeric, CGHseg.thres = -0.05)
ace.out <- pSegmentACE(cghE1[, 5:7], chrom.numeric)
wave.out <- pSegmentWavelets(cghE1[, 5:7], chrom.numeric)
wave.nm.out <- pSegmentWavelets(cghE1[, 5:7], chrom.numeric, mergeSegs = FALSE)
cbs.out <- pSegmentDNAcopy(cghE1[, 5:7], chrom.numeric)
cbs.ns.out <- pSegmentDNAcopy(cghE1[, 5:7], chrom.numeric, smooth = FALSE)
## Next onew we try, as it could fail when it should not.
try(psw.pos.out <- pSegmentPSW(cghE1[, 5:7], chrom.numeric, sign = 1))
try(psw.neg.out <- pSegmentPSW(cghE1[, 5:7], chrom.numeric, sign = -1))

## BioHMM is the only one that uses distances
## it is the slowest, so do only two
biohmm.out <- pSegmentBioHMM(cghE1[, 5:6], chrom.numeric, cghE1$UG.Start)

## With ACE we need to choose fdr
ace.out.sum <- summary(ace.out)

#### Writing
common <- cghE1[, 1:4]
writeResults(hmm.out, cghE1[, 5:7], common)
writeResults(glad.out, cghE1[, 5:7], common)        
writeResults(cghseg.out, cghE1[, 5:7], common)   
writeResults(ace.out.sum, cghE1[, 5:7], common)
writeResults(wave.out, cghE1[, 5:7], common)
writeResults(wave.nm.out, cghE1[, 5:7], common)
writeResults(cbs.out, cghE1[, 5:7], common)
try(writeResults(psw.pos.out, common))
try(writeResults(psw.neg.out, common))
writeResults(biohmm.out, cghE1[, 5:6], common)



### plot all

## Not run: 
##D 
##D ## This will not work unless you have a working Python installation
##D 
##D segmentPlot(hmm.out, 
##D             geneNames = cghE1[, 1],
##D             chrom.numeric = chrom.numeric,
##D             cghdata = cghE1[, 5:7],
##D             idtype = "ug",
##D             organism = "Hs")
##D 
##D segmentPlot(glad.out,
##D             geneNames = cghE1[, 1],
##D             chrom.numeric = chrom.numeric,
##D             cghdata = cghE1[, 5:7],
##D             idtype = "ug",
##D             organism = "Hs")
##D 
##D segmentPlot(cghseg.out,
##D             geneNames = cghE1[, 1],
##D             chrom.numeric = chrom.numeric,
##D             cghdata = cghE1[, 5:7],
##D             idtype = "ug",
##D             organism = "Hs")
##D 
##D segmentPlot(wave.out, 
##D             geneNames = cghE1[, 1],
##D             chrom.numeric = chrom.numeric,
##D             cghdata = cghE1[, 5:7],
##D             idtype = "ug",
##D             organism = "Hs")
##D 
##D segmentPlot(wave.nm.out,
##D             geneNames = cghE1[, 1],
##D             chrom.numeric = chrom.numeric,
##D             cghdata = cghE1[, 5:7],
##D             idtype = "ug",
##D             organism = "Hs")
##D 
##D segmentPlot(cbs.out, 
##D             geneNames = cghE1[, 1],
##D             chrom.numeric = chrom.numeric,
##D             cghdata = cghE1[, 5:7],
##D             idtype = "ug",
##D             organism = "Hs")
##D 
##D segmentPlot(psw.pos.out,
##D             geneNames = cghE1[, 1],
##D             chrom.numeric = chrom.numeric,
##D             cghdata = cghE1[, 5:7],
##D             idtype = "ug",
##D             organism = "Hs")
##D 
##D segmentPlot(psw.neg.out,
##D             geneNames = cghE1[, 1],
##D             chrom.numeric = chrom.numeric,
##D             cghdata = cghE1[, 5:7],
##D             idtype = "ug",
##D             organism = "Hs")
##D 
##D segmentPlot(biohmm.out, 
##D             geneNames = cghE1[, 1],
##D             chrom.numeric = chrom.numeric,
##D             cghdata = cghE1[, 5:6],
##D             idtype = "ug",
##D             organism = "Hs")
##D segmentPlot(ace.out.sum, 
##D             geneNames = cghE1[, 1],
##D             chrom.numeric = chrom.numeric,
##D             cghdata = cghE1[, 5:7],
##D             idtype = "ug",
##D             organism = "Hs")
##D 
## End(Not run)

## To verify we are doing the right thing with Picard et al.'s method,
## compare with their figure 1. Need to get the txt files with the
## original data.
## We got the data from the GLAD BioC package, and then exported the txt.
## For convenience, it is in the "/inst/txt-data" directory of our package.

## This verifies the output (look at ADaCGH.R for further details and other
## comparisons):

## coriel.data <- read.table("gm03563.txt", header = TRUE)
## cd3 <- coriel.data[coriel.data$Chromosome == 3, 3]
    
## out.our <- CGHsegWrapper(cd3, rep(1, length(cd3)), s = -0.5)  ## k = 2
## out.our

## so our implementation seems correct. Look at where the
## breakpoint is located, etc, and it is like figure 1 of
## Picard's paper.






